"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.sumZero_multiplePointers = exports.sumZero = exports.sameSquared_frequency = exports.sameSquared = void 0;
/*
  Compares if array2 has content of array1 squared
  Naive
  O(n ^2)
*/
function sameSquared(arr1, arr2) {
    if (arr1.length !== arr2.length)
        return false;
    for (let i = 0; i < arr1.length; i++) {
        let correctIndex = arr2.indexOf(Math.pow(arr1[i], 2));
        if (correctIndex === -1)
            return false;
        arr2.splice(correctIndex, 1);
    }
    return true;
}
exports.sameSquared = sameSquared;
/*
  Compares if array2 has content of array1 squared
  Frequency Pattern
  O(n)
*/
function sameSquared_frequency(arr1, arr2) {
    if (arr1.length !== arr2.length)
        return false;
    const frequencyCounter1 = {};
    const frequencyCounter2 = {};
    for (let val of arr1) {
        frequencyCounter1[val] = (frequencyCounter1[val] || 0) + 1;
    }
    for (let val of arr2) {
        frequencyCounter2[val] = (frequencyCounter2[val] || 0) + 1;
    }
    for (let key in frequencyCounter1) {
        if (!(Math.pow(Number(key), 2) in frequencyCounter2))
            return false;
        if (frequencyCounter2[Math.pow(Number(key), 2)] !== frequencyCounter1[key])
            return false;
    }
    return true;
}
exports.sameSquared_frequency = sameSquared_frequency;
/*
  Naive comparison
  Time complexity: O(n^2)
  Space complexity: O(1)
*/
function sumZero(arr) {
    for (let i = 0; i < arr.length; i++) {
        for (let j = i + 1; j < arr.length; j++) {
            if (arr[i] + arr[j] === 0) {
                return [arr[i], arr[j]];
            }
        }
    }
    return [];
}
exports.sumZero = sumZero;
/*
  Implementing multiple pointers pattern
  Time complexity: O(n)
  Space complexity: O(1)
*/
function sumZero_multiplePointers(arr) {
    let left = 0;
    let right = arr.length - 1;
    while (left < right) {
        let sum = arr[left] + arr[right];
        if (sum === 0) {
            return [arr[left], arr[right]];
        }
        else if (sum > 0) {
            right--;
        }
        else {
            left++;
        }
    }
}
exports.sumZero_multiplePointers = sumZero_multiplePointers;
