"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Node_1 = __importDefault(require("./Node"));
class DoublyLinkedList {
    constructor() {
        this.head = null;
        this.tail = null;
        this.length = 0;
    }
    push(val) {
        const newNode = new Node_1.default(val);
        if (this.length === 0) {
            this.head = newNode;
            this.tail = newNode;
        }
        else {
            if (this.tail) {
                this.tail.next = newNode;
            }
            newNode.prev = this.tail;
            this.tail = newNode;
        }
        this.length++;
        return this;
    }
    pop() {
        if (this.length === 0)
            return undefined;
        const tail = this.tail;
        if (this.length === 1) {
            this.head = null;
            this.tail = null;
        }
        else {
            if (this.tail)
                if (tail && tail.prev) {
                    this.tail = tail === null || tail === void 0 ? void 0 : tail.prev;
                    this.tail.next = null;
                    tail.prev = null;
                }
        }
        this.length--;
        return tail;
    }
    shift() {
        if (this.head === null)
            return undefined;
        let oldHead = this.head;
        if (this.length === 1) {
            this.head = null;
            this.tail = null;
        }
        else {
            this.head = oldHead.next;
            if (this.head) {
                this.head.prev = null;
            }
            oldHead.next = null;
        }
        this.length--;
        return oldHead;
    }
    unshift(val) {
        const newNode = new Node_1.default(val);
        if (this.head === null) {
            this.head = newNode;
            this.tail = newNode;
        }
        else {
            this.head.prev = newNode;
            newNode.next = this.head;
            this.head = newNode;
        }
        this.length++;
        return this;
    }
    get(n) {
        if (n < 0 || n >= this.length || this.head === null || this.tail === null)
            return undefined;
        let count;
        let current;
        const isInFirstHalf = n <= this.length / 2;
        if (isInFirstHalf) {
            // Loop forward
            count = 0;
            current = this.head;
            while (count !== n) {
                current = current === null || current === void 0 ? void 0 : current.next;
                count++;
            }
        }
        else {
            // Loop backward
            count = this.length - 1;
            current = this.tail;
            while (count !== n) {
                current = current === null || current === void 0 ? void 0 : current.prev;
                count--;
            }
        }
        return current;
    }
    set(n, val) {
        const foundNode = this.get(n);
        if (foundNode !== null && foundNode !== undefined) {
            foundNode.val = val;
            return true;
        }
        return false;
    }
}
exports.default = DoublyLinkedList;
