"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const DoublyLinkedList_1 = __importDefault(require("./DoublyLinkedList"));
describe("DoublyLinkedList", () => {
    describe("push", () => {
        it("adds a Node to the end of the List", () => {
            var _a, _b;
            const list = new DoublyLinkedList_1.default();
            list.push("value one");
            list.push("value two");
            expect((_a = list.tail) === null || _a === void 0 ? void 0 : _a.val).toEqual("value two");
            list.push("value three");
            expect((_b = list.tail) === null || _b === void 0 ? void 0 : _b.val).toEqual("value three");
        });
        it("adds Node when the List is previously empty", () => {
            var _a, _b;
            const list = new DoublyLinkedList_1.default();
            list.push("value one");
            expect((_a = list.head) === null || _a === void 0 ? void 0 : _a.val).toEqual("value one");
            expect((_b = list.tail) === null || _b === void 0 ? void 0 : _b.val).toEqual("value one");
        });
        it("increments length by one", () => {
            const list = new DoublyLinkedList_1.default();
            expect(list.length).toEqual(0);
            list.push("value one");
            expect(list.length).toEqual(1);
            list.push("value two");
            expect(list.length).toEqual(2);
        });
    });
    describe("pop", () => {
        it("removes Node at end of the List", () => {
            var _a;
            const list = new DoublyLinkedList_1.default();
            list.push("value one");
            expect((_a = list.tail) === null || _a === void 0 ? void 0 : _a.val).toEqual("value one");
            list.pop();
            expect(list.tail).toEqual(null);
        });
        it("returns undefined if List is empty", () => {
            const list = new DoublyLinkedList_1.default();
            expect(list.head).toEqual(null);
            const result = list.pop();
            expect(result).toEqual(undefined);
        });
        it("decrements length by one", () => {
            const list = new DoublyLinkedList_1.default();
            list.push("value one");
            expect(list.length).toEqual(1);
            list.pop();
            expect(list.length).toEqual(0);
        });
        it("returns the removed Node", () => {
            const list = new DoublyLinkedList_1.default();
            list.push("value one");
            list.push("value two");
            const result = list.pop();
            expect(result).toEqual({
                next: null,
                prev: null,
                val: "value two",
            });
        });
    });
    describe("shift", () => {
        it("removes Node at beginning of the List", () => {
            var _a, _b;
            const list = new DoublyLinkedList_1.default();
            list.push("value one");
            list.push("value two");
            expect((_a = list.head) === null || _a === void 0 ? void 0 : _a.val).toEqual("value one");
            list.shift();
            expect((_b = list.head) === null || _b === void 0 ? void 0 : _b.val).toEqual("value two");
        });
        it("returns undefined if List is empty", () => {
            const list = new DoublyLinkedList_1.default();
            const result = list.shift();
            expect(result).toEqual(undefined);
        });
        it("decrements length by one", () => {
            const list = new DoublyLinkedList_1.default();
            list.push("value one");
            expect(list.length).toEqual(1);
            list.shift();
            expect(list.length).toEqual(0);
        });
        it("returns the removed Node", () => {
            const list = new DoublyLinkedList_1.default();
            list.push("value one");
            const result = list.shift();
            expect(result).toEqual({ next: null, val: "value one" });
        });
    });
    describe("unshift", () => {
        it("adds a Node at the beginning of the List", () => {
            var _a, _b;
            const list = new DoublyLinkedList_1.default();
            list.push("value one");
            expect((_a = list.head) === null || _a === void 0 ? void 0 : _a.val).toEqual("value one");
            list.unshift("value two");
            expect((_b = list.head) === null || _b === void 0 ? void 0 : _b.val).toEqual("value two");
        });
        it("adds a Node when the List is previously empty", () => {
            var _a, _b;
            const list = new DoublyLinkedList_1.default();
            list.unshift("value one");
            expect((_a = list.head) === null || _a === void 0 ? void 0 : _a.val).toEqual("value one");
            expect((_b = list.tail) === null || _b === void 0 ? void 0 : _b.val).toEqual("value one");
        });
        it("increments length by one", () => {
            const list = new DoublyLinkedList_1.default();
            expect(list.length).toEqual(0);
            list.unshift("value one");
            expect(list.length).toEqual(1);
        });
    });
    describe("get", () => {
        it("returns the Node at place n", () => {
            const list = new DoublyLinkedList_1.default();
            list.push("value one");
            list.push("value two");
            list.push("value three");
            const result = list.get(1);
            expect(result === null || result === void 0 ? void 0 : result.val).toEqual("value two");
        });
        it("returns undefined if n does not exist", () => {
            const list = new DoublyLinkedList_1.default();
            expect(list.get(0)).toEqual(undefined);
            expect(list.get(-1)).toEqual(undefined);
        });
        it("returns Node at the first half of of the List", () => {
            const list = new DoublyLinkedList_1.default();
            list.push("value one");
            list.push("value two");
            list.push("value three");
            const result = list.get(0);
            expect(result === null || result === void 0 ? void 0 : result.val).toEqual("value one");
        });
        it("returns Node at the second half of of the List", () => {
            const list = new DoublyLinkedList_1.default();
            list.push("value one");
            list.push("value two");
            list.push("value three");
            const result = list.get(2);
            expect(result === null || result === void 0 ? void 0 : result.val).toEqual("value three");
        });
    });
    describe("set", () => {
        it("updates the value at place n", () => {
            var _a;
            const list = new DoublyLinkedList_1.default();
            list.push("value one");
            list.push("value two");
            list.push("value three");
            list.set(2, "new value");
            expect((_a = list.tail) === null || _a === void 0 ? void 0 : _a.val).toEqual("new value");
        });
        it("returns false if no value exist at place n", () => {
            const list = new DoublyLinkedList_1.default();
            expect(list.set(0, "new value")).toEqual(false);
        });
    });
    describe("insert", () => { });
    describe("remove", () => { });
});
