"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const SinglyLinkedList_1 = __importDefault(require("./SinglyLinkedList"));
describe("SinglyLinkedList", () => {
    describe("push", () => {
        it("takes a value and adds to the end of the List", () => {
            var _a;
            const list = new SinglyLinkedList_1.default();
            const val = "hello";
            list.push(val);
            expect((_a = list.tail) === null || _a === void 0 ? void 0 : _a.val).toEqual(val);
        });
        it("if the value is the first added, then both head and tail will be the same Node", () => {
            var _a, _b;
            const list = new SinglyLinkedList_1.default();
            const val = "hello";
            list.push(val);
            expect((_a = list.head) === null || _a === void 0 ? void 0 : _a.val).toEqual(val);
            expect((_b = list.tail) === null || _b === void 0 ? void 0 : _b.val).toEqual(val);
        });
        it("if the value is NOT the first value added, then the head and tail will NOT be the same Node", () => {
            var _a, _b;
            const list = new SinglyLinkedList_1.default();
            const val1 = "hello";
            const val2 = "people";
            list.push(val1);
            list.push(val2);
            expect((_a = list.head) === null || _a === void 0 ? void 0 : _a.val).toEqual(val1);
            expect((_b = list.tail) === null || _b === void 0 ? void 0 : _b.val).toEqual(val2);
        });
        it("increments the length by one", () => {
            const list = new SinglyLinkedList_1.default();
            expect(list.length).toEqual(0);
            list.push("value one");
            expect(list.length).toEqual(1);
        });
    });
    describe("pop", () => {
        it("returns the last Node from the List", () => {
            const list = new SinglyLinkedList_1.default();
            list.push("value one");
            list.push("value two");
            list.push("value three");
            const value = list.pop();
            expect(value).toEqual({ next: null, val: "value three" });
        });
        it("removes the last Node from the List", () => {
            var _a, _b, _c, _d;
            const list = new SinglyLinkedList_1.default();
            list.push("value one");
            list.push("value two");
            list.push("value three");
            expect((_a = list.head) === null || _a === void 0 ? void 0 : _a.val).toEqual("value one");
            expect((_b = list.tail) === null || _b === void 0 ? void 0 : _b.val).toEqual("value three");
            list.pop();
            expect((_c = list.head) === null || _c === void 0 ? void 0 : _c.val).toEqual("value one");
            expect((_d = list.tail) === null || _d === void 0 ? void 0 : _d.val).toEqual("value two");
        });
        it("decrements the length by one", () => {
            const list = new SinglyLinkedList_1.default();
            list.push("value one");
            list.push("value two");
            list.push("value three");
            expect(list.length).toEqual(3);
            list.pop();
            expect(list.length).toEqual(2);
        });
        it("handles when there is only one Node left", () => {
            var _a, _b;
            const list = new SinglyLinkedList_1.default();
            list.push("value one");
            const value = list.pop();
            expect(value).toEqual({ next: null, val: "value one" });
            expect((_a = list.head) === null || _a === void 0 ? void 0 : _a.val).toEqual(undefined);
            expect((_b = list.tail) === null || _b === void 0 ? void 0 : _b.val).toEqual(undefined);
            expect(list.length).toEqual(0);
        });
    });
    describe("shift", () => {
        it("returns the first Node from the List", () => {
            const list = new SinglyLinkedList_1.default();
            list.push("value one");
            const value = list.shift();
            expect(value).toEqual({ next: null, val: "value one" });
        });
        it("removes the first Node from the List", () => {
            const list = new SinglyLinkedList_1.default();
            list.push("value one");
            list.shift();
        });
        it("decrements the length by one", () => {
            const list = new SinglyLinkedList_1.default();
            list.push("value one");
            expect(list.length).toEqual(1);
            list.shift();
            expect(list.length).toEqual(0);
        });
        it("returns undefined if there is no Node", () => { });
    });
    describe("unshift", () => {
        it("adds a new Node with the value given at the start of the List", () => {
            var _a, _b;
            const list = new SinglyLinkedList_1.default();
            list.push("value one");
            list.unshift("value two");
            expect((_a = list.head) === null || _a === void 0 ? void 0 : _a.val).toEqual("value two");
            expect((_b = list.tail) === null || _b === void 0 ? void 0 : _b.val).toEqual("value one");
        });
        it("sets head and tail to new Node if there are no previous Nodes", () => {
            var _a, _b, _c, _d;
            const list = new SinglyLinkedList_1.default();
            list.unshift("value two");
            expect((_a = list.head) === null || _a === void 0 ? void 0 : _a.val).toEqual("value two");
            expect((_b = list.head) === null || _b === void 0 ? void 0 : _b.next).toEqual(null);
            expect((_c = list.tail) === null || _c === void 0 ? void 0 : _c.val).toEqual("value two");
            expect((_d = list.tail) === null || _d === void 0 ? void 0 : _d.next).toEqual(null);
        });
        it("increments the length by one", () => {
            const list = new SinglyLinkedList_1.default();
            expect(list.length).toEqual(0);
            list.unshift("value one");
            expect(list.length).toEqual(1);
        });
    });
    describe("get", () => {
        it("returns the (n)th item in the List", () => {
            const list = new SinglyLinkedList_1.default();
            list.push("value one");
            list.push("value two");
            list.push("value three");
            list.push("value four");
            list.push("value five");
            list.push("value six");
            list.push("value seven");
            list.push("value eight");
            const item = list.get(5);
            expect(item === null || item === void 0 ? void 0 : item.val).toEqual("value six");
        });
        it("returns undefined if there is no item", () => {
            const list = new SinglyLinkedList_1.default();
            const item = list.get(1);
            expect(item).toEqual(undefined);
        });
    });
    describe("set", () => {
        it("updates the value of item", () => {
            var _a;
            const list = new SinglyLinkedList_1.default();
            list.push("value one");
            list.set(0, "new value");
            expect((_a = list.head) === null || _a === void 0 ? void 0 : _a.val).toEqual("new value");
        });
        it("returns true if value is updated", () => {
            const list = new SinglyLinkedList_1.default();
            list.push("value one");
            const result = list.set(0, "new value");
            expect(result).toEqual(true);
        });
        it("returns false if there is no item to update", () => {
            const list = new SinglyLinkedList_1.default();
            const item = list.set(1, "new value");
            expect(item).toEqual(false);
        });
    });
    describe("insert", () => {
        it("returns false if n is less than zero", () => {
            const list = new SinglyLinkedList_1.default();
            expect(list.insert(-1, "value")).toEqual(false);
        });
        it("returns false if n is greater than length", () => {
            const list = new SinglyLinkedList_1.default();
            expect(list.insert(1, "value")).toEqual(false);
        });
        it("unshift a new Node to the start of the List if n is 0", () => {
            var _a;
            const list = new SinglyLinkedList_1.default();
            list.push("value one");
            list.push("value two");
            list.push("value three");
            list.push("value four");
            list.insert(0, "new value");
            expect((_a = list.head) === null || _a === void 0 ? void 0 : _a.val).toEqual("new value");
        });
        it("pushes a new Node to the end of the List if n same as length", () => {
            var _a;
            const list = new SinglyLinkedList_1.default();
            list.push("value one");
            list.push("value two");
            list.push("value three");
            list.push("value four");
            list.insert(4, "new value");
            expect((_a = list.tail) === null || _a === void 0 ? void 0 : _a.val).toEqual("new value");
        });
        it("increments the length by one", () => {
            const list = new SinglyLinkedList_1.default();
            expect(list.length).toEqual(0);
            list.insert(0, "new value");
            expect(list.length).toEqual(1);
        });
        it("returns true if insertion is successful", () => {
            const list = new SinglyLinkedList_1.default();
            expect(list.insert(0, "new value")).toEqual(true);
        });
    });
    describe("remove", () => {
        it("returns undefined if n is less than zero", () => {
            const list = new SinglyLinkedList_1.default();
            expect(list.remove(-1)).toEqual(undefined);
        });
        it("returns undefined if n is greater than length", () => {
            const list = new SinglyLinkedList_1.default();
            expect(list.remove(0)).toEqual(undefined);
        });
        it("returns first item", () => {
            var _a;
            const list = new SinglyLinkedList_1.default();
            list.push("value one");
            list.push("value two");
            list.push("value three");
            list.push("value four");
            expect((_a = list.remove(0)) === null || _a === void 0 ? void 0 : _a.val).toEqual("value one");
        });
        it("returns last item", () => {
            const list = new SinglyLinkedList_1.default();
            list.push("value one");
            list.push("value two");
            list.push("value three");
            list.push("value four");
            expect(list.remove(3)).toEqual({ next: null, val: "value four" });
        });
        it("returns item in list that is not first or last", () => {
            var _a;
            const list = new SinglyLinkedList_1.default();
            list.push("value one");
            list.push("value two");
            list.push("value three");
            list.push("value four");
            expect((_a = list.remove(2)) === null || _a === void 0 ? void 0 : _a.val).toEqual("value three");
        });
    });
    describe("reverse", () => {
        it("returns the list in reversed order", () => {
            const list = new SinglyLinkedList_1.default();
            list.push("value one");
            list.push("value two");
            list.push("value three");
            list.push("value four");
            const listOriginal = {
                head: {
                    next: {
                        next: {
                            next: { next: null, val: "value four" },
                            val: "value three",
                        },
                        val: "value two",
                    },
                    val: "value one",
                },
                length: 4,
                tail: { next: null, val: "value four" },
            };
            expect(list).toEqual(listOriginal);
            list.reverse();
            const reversedList = {
                head: {
                    next: {
                        next: {
                            next: {
                                next: null,
                                val: "value one",
                            },
                            val: "value two",
                        },
                        val: "value three",
                    },
                    val: "value four",
                },
                length: 4,
                tail: {
                    next: null,
                    val: "value one",
                },
            };
            expect(list).toEqual(reversedList);
        });
    });
});
