"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.shellSort_bound = exports.shellSort = exports.insertionSort = void 0;
/*
 * Insertion sort
 * - Complex
 * + Good for nearly sorted arrays or where new data is continuously added
 *
 * Time Complexity:
 *   Best: Ω(n)
 *   Average Θ(n^2)
 *   Worst: O(n^2)
 *
 * Space complexity: O(1)
 *
 * Comparison (will vary, use for relative comparison only)
 *   100 items 0.209ms
 *   1000 items 2.91ms
 *   10000 items 39.276ms
 *   100000 items 3.137 seconds (!)
 *   1000000 items 5 min 49.567 seconds (!)
 */
function insertionSort(arr) {
    for (let i = 1, l = arr.length; i < l; i++) {
        let tmp = arr[i];
        for (var j = i - 1; j >= 0; j--) {
            if (arr[j] <= tmp)
                break;
            arr[j + 1] = arr[j];
        }
        arr[j + 1] = tmp;
    }
    return arr;
}
exports.insertionSort = insertionSort;
/*
 * Shell Sort
 * In-place
 *
 * Shell sort is an optimized version of Insertion sort,
 * that basically allows the exchange of items that are far
 * away from another.
 *
 * Time Complexity:
 *   Best: Ω(n log(n))
 *   Average Θ(n(log(n))^2)
 *   Worst: O(n(log(n))^2)
 *
 * Space complexity: O(1)
 *
 * Comparison (will vary, use for relative comparison only)
 *   100 items 0.155ms
 *   1000 items 3.176ms
 *   10000 items 6.098ms
 *   100000 items 31.106ms
 *   1000000 items 256.023ms
 */
function shellSort(arr) {
    let gap = Math.round(arr.length / 2);
    let gapIndex;
    while (gap > 0) {
        for (gapIndex = gap; gapIndex < arr.length; gapIndex++) {
            let tmp = arr[gapIndex];
            let j = gapIndex;
            while (j >= gap && arr[j - gap] > tmp) {
                arr[j] = arr[j - gap];
                j -= gap;
            }
            arr[j] = tmp;
        }
        gap = Math.round(gap / 2.2);
    }
    return arr;
}
exports.shellSort = shellSort;
/**
 * Shell Sort - Bound
 * Same as Shell Sort, but allows for setting a boundary which
 * helps performance in very long arrays, if you know which part
 * of it that needs sorting.
 */
function shellSort_bound(arr, start, end) {
    let gap = Math.round((start + end) / 2);
    let gapIndex;
    while (gap >= start) {
        for (gapIndex = gap; gapIndex < end; gapIndex++) {
            let tmp = arr[gapIndex];
            let j = gapIndex;
            while (j >= gap && arr[j - gap] > tmp) {
                arr[j] = arr[j - gap];
                j -= gap;
            }
            arr[j] = tmp;
        }
        gap = Math.round(gap / 2.2);
    }
    return arr;
}
exports.shellSort_bound = shellSort_bound;
exports.default = {
    insertionSort,
    shellSort,
    shellSort_bound,
};
