"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.heapSort_recursive = exports.heapSort = exports.selectionSort = void 0;
const sorterHelpers_1 = require("./sorterHelpers");
/*
 * Selection Sort
 * Swapping, from biggest to smallest
 *
 * + Simple, good for small data sets
 * - Not good for nearly sorted lists
 *
 * Time Complexity Average: O(n^2)
 */
function selectionSort(arr) {
    for (let i = 0; i < arr.length; i++) {
        let lowest = i;
        for (let j = i + 1; j < arr.length; j++) {
            if (arr[j] < arr[lowest]) {
                lowest = j;
            }
        }
        if (i !== lowest) {
            sorterHelpers_1.swap({ array: arr, index1: i, index2: lowest });
        }
    }
    return arr;
}
exports.selectionSort = selectionSort;
//selectionSort([34, 22, 10, 19, 17])
/*
 * Heap sort
 * In-place
 * None-stable
 * Max-heap
 *
 * Time Complexity:
 *   Best: Ω(n log(n))
 *   Average Θ(n log(n))
 *   Worst: O(n log(n))
 *
 * Space complexity: O(1)
 *
 * Comparison (will vary, use for relative comparison only)
 *   100 items 0.256ms
 *   1000 items 2.465ms
 *   10000 items 6.639ms
 *   100000 items 22.916ms
 *   1000000 items 180.906ms
 */
function heapSort(arr) {
    sorterHelpers_1.heapify(arr);
    for (var end = arr.length - 1; end > 0; end--) {
        sorterHelpers_1.swap({ array: arr, index1: end, index2: 0 });
        sorterHelpers_1.shiftDown(arr, 0, end - 1);
    }
    return arr;
}
exports.heapSort = heapSort;
/*
 * Heap sort - recursive
 *
 * Same as above, but uses recursion to create the max-heap.
 * As it is done in-place (not creating a new instance of the
 * array) it doesn't affect memory usage negatively.
 */
function heapSort_recursive(arr) {
    const length = arr.length;
    let lastParentNode = Math.floor(length / 2 - 1);
    let lastChild = length - 1;
    while (lastParentNode >= 0) {
        sorterHelpers_1.heapify_recursive(arr, length, lastParentNode);
        lastParentNode--;
    }
    while (lastChild >= 0) {
        // Swap
        [arr[0], arr[lastChild]] = [arr[lastChild], arr[0]];
        sorterHelpers_1.heapify_recursive(arr, lastChild, 0);
        lastChild--;
    }
    return arr;
}
exports.heapSort_recursive = heapSort_recursive;
exports.default = {
    selectionSort,
    heapSort,
    heapSort_recursive,
};
