"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.newGap = exports.findMostDigitsFromNumbers = exports.digitCount = exports.getDigit = exports.heapify_recursive = exports.shiftDown = exports.heapify = exports.pivot = exports.swap = void 0;
function swap({ array, index1, index2 }) {
    const tmp = array[index1];
    array[index1] = array[index2];
    array[index2] = tmp;
}
exports.swap = swap;
function pivot({ array, start = 0, end = array.length + 1, }) {
    let pivotPoint = array[start];
    let swapIndex = start;
    for (let i = start + 1; i < end; i++) {
        if (pivotPoint > array[i]) {
            swapIndex++;
            swap({ array, index1: swapIndex, index2: i });
        }
    }
    swap({ array, index1: start, index2: swapIndex });
    return swapIndex;
}
exports.pivot = pivot;
/**
 * Heapify
 * Traverses the elements of an array (from back to front)
 * and uses shiftDown to arrange the numbers according to an
 * max heap structure.
 */
function heapify(arr) {
    for (let start = (arr.length >> 1) - 1; start >= 0; start--) {
        shiftDown(arr, start, arr.length - 1);
    }
}
exports.heapify = heapify;
/**
 * Shift Down
 * Traverses an set of an array and organizes the numbers in
 * the order of a max heap structure (parent larger than
 * children).
 */
function shiftDown(arr, start, end) {
    let root = start;
    while (root * 2 + 1 <= end) {
        let leftChild = root * 2 + 1;
        let s = root;
        if (arr[s] < arr[leftChild]) {
            s = leftChild;
        }
        const rightChild = leftChild + 1;
        if (rightChild <= end && arr[s] < arr[rightChild]) {
            s = rightChild;
        }
        if (s !== root) {
            swap({ array: arr, index1: root, index2: s });
            root = s;
        }
        else {
            return;
        }
    }
}
exports.shiftDown = shiftDown;
/**
 * Heapify Recursive
 * Does the same as heapify and shiftDown together, but
 * in a recursive manner
 */
function heapify_recursive(arr, length, i) {
    let largest = i;
    let left = i * 2 + 1;
    let right = left + 1;
    if (left < length && arr[left] > arr[largest]) {
        largest = left;
    }
    if (right < length && arr[right] > arr[largest]) {
        largest = right;
    }
    if (largest != i) {
        // Swap
        [arr[i], arr[largest]] = [arr[largest], arr[i]];
        heapify_recursive(arr, length, largest);
    }
    return arr;
}
exports.heapify_recursive = heapify_recursive;
function getDigit({ num, place }) {
    return Math.floor(Math.abs(num) / Math.pow(10, place)) % 10;
}
exports.getDigit = getDigit;
/**
 * Digit Count
 * Counts the number of digits in a number
 */
function digitCount(num) {
    if (num === 0)
        return 1;
    return Math.floor(Math.log10(Math.abs(num))) + 1;
}
exports.digitCount = digitCount;
/**
 * Most Digits
 * Finds the number with the most digits in an array
 */
function findMostDigitsFromNumbers(array) {
    let maxDigits = 0;
    for (let i = 0; i < array.length; i++) {
        maxDigits = Math.max(maxDigits, digitCount(array[i]));
    }
    return maxDigits;
}
exports.findMostDigitsFromNumbers = findMostDigitsFromNumbers;
function newGap(gap) {
    gap = ((gap * 10) / 13) | 0;
    if (gap == 9 || gap == 10)
        gap = 11;
    if (gap < 1)
        gap = 1;
    return gap;
}
exports.newGap = newGap;
