"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const sorterHelpers_1 = require("./sorterHelpers");
describe("Helpers", () => {
    describe("swap", () => {
        it("will swap places of two numbers", () => {
            const arr = [1, 3];
            const swapped = [3, 1];
            sorterHelpers_1.swap({ array: arr, index1: 0, index2: 1 });
            expect(arr).toEqual(swapped);
        });
    });
    describe("pivot", () => {
        it("swaps items smaller than the starting pivot value to the left and then returns the new pivot index", () => {
            const arr = [5, 2, 1, 8, 4, 7, 6, 3];
            const newPivotIndex = sorterHelpers_1.pivot({ array: arr, start: 0 });
            expect(arr).toEqual([3, 2, 1, 4, 5, 7, 6, 8]);
            expect(newPivotIndex).toEqual(4);
        });
    });
    describe("getDigit", () => {
        it("returns digit at place in number", () => {
            const num = 12345;
            // places that doesn't exist returns 0
            expect(sorterHelpers_1.getDigit({ num, place: -1 })).toEqual(0);
            // place 0 at 1st
            expect(sorterHelpers_1.getDigit({ num, place: 0 })).toEqual(5);
            // place 1 at 10th
            expect(sorterHelpers_1.getDigit({ num, place: 1 })).toEqual(4);
            // place 2 at 100ed
            expect(sorterHelpers_1.getDigit({ num, place: 2 })).toEqual(3);
            // place 3 at 1000nd
            expect(sorterHelpers_1.getDigit({ num, place: 3 })).toEqual(2);
            // place 4 at 10000nd
            expect(sorterHelpers_1.getDigit({ num, place: 4 })).toEqual(1);
            // place 5 doesn't exist, returns 0
            expect(sorterHelpers_1.getDigit({ num, place: 5 })).toEqual(0);
            // place 6 doesn't exist, returns 0
            expect(sorterHelpers_1.getDigit({ num, place: 6 })).toEqual(0);
        });
    });
    describe("digitCount", () => {
        it("counts the number of digits in a number", () => {
            expect(sorterHelpers_1.digitCount(-31)).toEqual(2);
            expect(sorterHelpers_1.digitCount(-2)).toEqual(1);
            expect(sorterHelpers_1.digitCount(5)).toEqual(1);
            expect(sorterHelpers_1.digitCount(51)).toEqual(2);
            expect(sorterHelpers_1.digitCount(151)).toEqual(3);
            expect(sorterHelpers_1.digitCount(1521)).toEqual(4);
            expect(sorterHelpers_1.digitCount(16521)).toEqual(5);
        });
    });
    describe("findMostDigitsFromNumbers", () => {
        it("finds number of digits in the largest number in an array of numbers", () => {
            const numbers = [12, 555, 16, 1677, 391, 10];
            // 1677 is largest and it has 4 digits
            expect(sorterHelpers_1.findMostDigitsFromNumbers(numbers)).toEqual(4);
        });
    });
    describe("shiftDown", () => {
        it("", () => {
            const numbers = [12, 555, 16, 1677, 391, 10];
            sorterHelpers_1.shiftDown(numbers, 0, 6);
            expect(numbers).toEqual([]);
        });
    });
});
